// Generated by CoffeeScript 1.7.1
(function() {
  "use strict";
  var Observable, Runtime, attachCleaner, bindEvent, bindObservable, bindSplat, createElement, dispose, elementCleaners, elementRefCounts, empty, eventNames, get, isEvent, isObject, makeElement, observeAttribute, observeAttributes, observeContent, release, retain, specialBindings, splat, valueBind, valueIndexOf;

  Observable = require("o_0");

  elementCleaners = new WeakMap;

  elementRefCounts = new WeakMap;

  retain = function(element) {
    var count;
    count = elementRefCounts.get(element) || 0;
    elementRefCounts.set(element, count + 1);
  };

  release = function(element) {
    var count;
    count = elementRefCounts.get(element) || 0;
    count--;
    if (count > 0) {
      elementRefCounts.set(element, count);
    } else {
      elementRefCounts["delete"](element);
      dispose(element);
    }
  };

  dispose = function(element) {
    var children, _ref;
    children = element.children;
    if (children != null) {
      Array.prototype.forEach.call(children, dispose);
    }
    if ((_ref = elementCleaners.get(element)) != null) {
      _ref.forEach(function(cleaner) {
        cleaner();
        elementCleaners["delete"](element);
      });
    }
  };

  attachCleaner = function(element, cleaner) {
    var cleaners;
    cleaners = elementCleaners.get(element);
    if (cleaners) {
      cleaners.push(cleaner);
    } else {
      elementCleaners.set(element, [cleaner]);
    }
  };

  eventNames = /^on(touch|animation|transition)(start|iteration|move|end|cancel)$/;

  isEvent = function(name, element) {
    return name.match(eventNames) || name in element;
  };

  valueBind = function(element, value, context) {
    switch (element.nodeName) {
      case "SELECT":
        element.oninput = element.onchange = function() {
          var optionValue, _ref, _value;
          _ref = this.children[this.selectedIndex], optionValue = _ref.value, _value = _ref._value;
          if (typeof value === "function") {
            value(_value || optionValue);
          }
        };
        bindObservable(element, value, context, function(newValue) {
          var options;
          element._value = newValue;
          if ((options = element._options)) {
            if (newValue.value != null) {
              element.value = (typeof newValue.value === "function" ? newValue.value() : void 0) || newValue.value;
            } else {
              element.selectedIndex = valueIndexOf(options, newValue);
            }
          } else {
            element.value = newValue;
          }
        });
        break;
      default:
        element.oninput = element.onchange = function() {
          if (typeof value === "function") {
            value(element.value);
          }
        };
        bindObservable(element, value, context, function(newValue) {
          if (element.value !== newValue) {
            element.value = newValue;
          }
        });
    }
  };

  specialBindings = {
    INPUT: {
      checked: function(element, value, context) {
        element.onchange = function() {
          if (typeof value === "function") {
            value(element.checked);
          }
        };
        return bindObservable(element, value, context, function(newValue) {
          element.checked = newValue;
        });
      }
    },
    SELECT: {
      options: function(element, values, context) {
        bindObservable(element, values, context, function(values) {
          empty(element);
          element._options = values;
          values.map(function(value, index) {
            var option, optionName, optionValue;
            option = createElement("option");
            option._value = value;
            if (isObject(value)) {
              optionValue = (value != null ? value.value : void 0) || index;
            } else {
              optionValue = value.toString();
            }
            bindObservable(option, optionValue, value, function(newValue) {
              option.value = newValue;
            });
            optionName = (value != null ? value.name : void 0) || value;
            bindObservable(option, optionName, value, function(newValue) {
              option.textContent = newValue;
            });
            element.appendChild(option);
            if (value === element._value) {
              element.selectedIndex = index;
            }
            return option;
          });
        });
      }
    }
  };

  observeAttribute = function(element, context, name, value) {
    var binding, nodeName, _ref;
    nodeName = element.nodeName;
    if (name === "value") {
      valueBind(element, value);
    } else if (binding = (_ref = specialBindings[nodeName]) != null ? _ref[name] : void 0) {
      binding(element, value, context);
    } else if (name.match(/^on/) && isEvent(name, element)) {
      bindEvent(element, name.substr(2), value, context);
    } else if (isEvent("on" + name, element)) {
      bindEvent(element, name, value, context);
    } else {
      bindObservable(element, value, context, function(newValue) {
        if ((newValue != null) && newValue !== false) {
          element.setAttribute(name, newValue);
        } else {
          element.removeAttribute(name);
        }
      });
    }
  };

  observeAttributes = function(element, context, attributes) {
    bindSplat(element, context, attributes, "id", function(ids) {
      var lastId;
      lastId = ids[ids.length - 1];
      if (ids.length) {
        element.id = lastId;
      } else {
        element.removeAttribute("id");
      }
    });
    bindSplat(element, context, attributes, "class", function(classes) {
      element.className = classes.join(" ");
    });
    bindSplat(element, context, attributes, "style", function(styles) {
      element.removeAttribute("style");
      styles.forEach(function(style) {
        if (isObject(style)) {
          return Object.assign(element.style, style);
        } else {
          return element.setAttribute("style", style);
        }
      });
    });
    Object.keys(attributes).forEach(function(name) {
      observeAttribute(element, context, name, attributes[name]);
    });
  };

  bindObservable = function(element, value, context, update) {
    var observable;
    observable = Observable(function() {
      update(get(value, context));
    });
    attachCleaner(element, observable.releaseDependencies);
  };

  bindEvent = function(element, name, fn, context) {
    if (typeof fn === "function") {
      element.addEventListener(name, fn.bind(context));
    }
  };

  bindSplat = function(element, context, attributes, key, fn) {
    var sources;
    sources = attributes[key];
    if (sources != null) {
      delete attributes[key];
      bindObservable(element, (function() {
        return splat(sources, context);
      }), context, fn);
    }
  };

  observeContent = function(element, context, contentFn) {
    var append, content;
    content = function() {
      var contents;
      contents = [];
      contentFn.call(context, {
        buffer: function(content) {
          contents.push(get(content, context));
        },
        element: makeElement
      });
      return contents;
    };
    append = function(item) {
      if (item == null) {

      } else if (typeof item.forEach === "function") {
        item.forEach(append);
      } else if (item instanceof Node) {
        retain(item);
        element.appendChild(item);
      } else {
        element.appendChild(document.createTextNode(item));
      }
    };
    bindObservable(element, content, context, function(contents) {
      empty(element);
      contents.forEach(append);
    });
  };

  makeElement = function(name, context, attributes, fn) {
    var element;
    element = createElement(name);
    observeAttributes(element, context, attributes);
    if (element.nodeName !== "SELECT") {
      observeContent(element, context, fn);
    }
    return element;
  };

  Runtime = function(context) {
    var self;
    self = {
      buffer: function(content) {
        if (self.root) {
          throw new Error("Cannot have multiple root elements");
        }
        self.root = content;
      },
      element: makeElement
    };
    return self;
  };

  Runtime.Observable = Observable;

  Runtime._elementCleaners = elementCleaners;

  Runtime._dispose = dispose;

  Runtime.retain = retain;

  Runtime.release = release;

  module.exports = Runtime;

  createElement = function(name) {
    return document.createElement(name);
  };

  empty = function(node) {
    var child;
    while (child = node.firstChild) {
      node.removeChild(child);
      release(child);
    }
  };

  isObject = function(x) {
    return typeof x === "object";
  };

  valueIndexOf = function(options, value) {
    if (isObject(value)) {
      return options.indexOf(value);
    } else {
      return options.map(function(option) {
        return option.toString();
      }).indexOf(value.toString());
    }
  };

  splat = function(sources, context) {
    return sources.map(function(source) {
      return get(source, context);
    }).reduce(function(a, b) {
      return a.concat(get(b));
    }, []).filter(function(x) {
      return x != null;
    });
  };

  get = function(x, context) {
    if (typeof x === 'function') {
      return x.call(context);
    } else {
      return x;
    }
  };

}).call(this);
